==> ec2init-0.11-py3-none-any.whl: ec2init/execution/executor.py <==
import logging
import time
from configparser import ConfigParser

from typing import Sequence, List, cast

from ec2init.execution.plan import Plan
from ec2init.execution.procedure import Procedure
from ec2init.execution.async_procedure import AsyncProcedure

logger = logging.getLogger(__name__)

class Executor:
    """
    Main Executor class to execute a Plan.
    The executor will keep running until
    the planner decides that no more Procedures
    need to be executed.
    """
    def __init__(self, name: str, config: ConfigParser) -> None:
        self.name = name
        self.max_retries = int(config.get('executor', 'max_procedure_retry', fallback=5))
        self.retries = 0
        self.config = config
        self.async_procedures: List[AsyncProcedure] = []
        self._start_time = time.time()

    def run(self, plan: Plan) -> None:
        """
        Execute the Plan until there are no more
        Procedures to execute.
        """
        while True:
            procedures = plan.create()
            if len(procedures) == 0:
                if len(self.async_procedures) != 0:
                        logger.info("Executor %s waiting for %d async procedures to finish", self.name, len(self.async_procedures))
                        to_remove = []
                        for p in self.async_procedures:
                            try:
                                p.wait_to_finish()
                                to_remove.append(p)
                            except Exception as ex:
                                if p.friendly_name is None:
                                    friendly_name = p.name
                                else:
                                    friendly_name = p.friendly_name
                                error_message = f"Error executing {friendly_name}"
                                plan.report_error(error_message)
                                for p in self.async_procedures:
                                    p.cancel()
                                raise ex
                        for p in to_remove:
                            self.async_procedures.remove(p)
                        # It is important to continue here and not let the loop break
                        # After we've waited for async Procedures to finish we might have
                        # procedures which will be added. We want to give the executor
                        # one more chance to go through the Plan
                        continue

                elapsed_time = time.time() - self._start_time
                logger.info("Executor %s took %d seconds to execute all procedures, exiting", self.name, elapsed_time)
                break
            
            try:
                self._react(plan, procedures)                
            except Exception as ex:
                for p in self.async_procedures:
                    p.cancel()
                raise ex    

            time.sleep(1)


    def _react(self, plan: Plan, procedures: Sequence[Procedure]) -> None:
        """
        Actually run the Procedures and their children if any.
        """
        failed = False
        for p in procedures:
            try:
                if p.is_async():
                    self.async_procedures.append(cast(AsyncProcedure, p))
                p.pre()
                inner_procedures = p.execute()
                self.retries = 0
                if len(inner_procedures) > 0:
                    self._react(plan, inner_procedures)
            except Exception as ex:
                if not failed:
                    self.retries += 1
                    failed = True
                logger.warning("%s Error executing procedure %s rolling back! Error: %s", self.name, p.name, ex)
                try:
                    p.rollback()
                except Exception as rex:
                    logger.warning("%s Error rolling back %s, continue with next procedure. Error: %s", self.name, p.name, rex)
                if p.fail_plan or self.retries >= self.max_retries:
                    logger.error("%s - Reached maximum number of retries. Exiting...", self.name)
                    if p.friendly_name is None:
                        friendly_name = p.name
                    else:
                        friendly_name = p.friendly_name
                    error_message = f"Error executing {friendly_name}"
                    plan.report_error(error_message)
                    raise ex

        for async_p in self.async_procedures:
            async_p.max_retries = self.max_retries
            try:
                async_p.check_unhandled_exception()
            except Exception as ex:
                if async_p.friendly_name is None:
                    friendly_name = async_p.name
                else:
                    friendly_name = async_p.friendly_name
                error_message = f"Error executing {friendly_name}"
                plan.report_error(error_message)
                raise ex

==> ec2init-0.17-py3-none-any.whl: ec2init/execution/executor.py <==
import logging
import time
from configparser import ConfigParser

from typing import Sequence, List, cast

from ec2init.execution.plan import Plan
from ec2init.execution.procedure import Procedure
from ec2init.execution.async_procedure import AsyncProcedure
from ec2init.utils.exceptions import Ec2InitFatalException, ConfigException

logger = logging.getLogger(__name__)

class Executor:
    """
    Main Executor class to execute a Plan.
    The executor will keep running until
    the planner decides that no more Procedures
    need to be executed.
    """
    def __init__(self, name: str, config: ConfigParser) -> None:
        self.name = name
        self.max_retries = int(config.get('executor', 'max_procedure_retry', fallback=5))
        self.retries = 0
        self.config = config
        self.async_procedures: List[AsyncProcedure] = []
        self._start_time = time.time()

    def cancelAllAsync(self):
        for p in self.async_procedures:
            p.cancel()

    def run(self, plan: Plan) -> None:
        """
        Execute the Plan until there are no more
        Procedures to execute.
        """
        while True:
            procedures = plan.create()
            if len(procedures) == 0:
                if len(self.async_procedures) != 0:
                        logger.info("Executor %s waiting for %d async procedures to finish", self.name, len(self.async_procedures))
                        to_remove = []
                        for p in self.async_procedures:
                            try:
                                p.wait_to_finish()
                                to_remove.append(p)
                            except ConfigException as ex:
                                plan.report_configuration_error(ex.errorMsg)
                                self.cancelAllAsync()
                                raise
                            except Exception as ex:
                                if p.friendly_name is None:
                                    friendly_name = p.name
                                else:
                                    friendly_name = p.friendly_name
                                error_message = f"Error executing {friendly_name}"
                                plan.report_error(error_message)
                                self.cancelAllAsync()
                                raise
                        for p in to_remove:
                            self.async_procedures.remove(p)
                        # It is important to continue here and not let the loop break
                        # After we've waited for async Procedures to finish we might have
                        # procedures which will be added. We want to give the executor
                        # one more chance to go through the Plan
                        continue

                elapsed_time = time.time() - self._start_time
                logger.info("Executor %s took %d seconds to execute all procedures, exiting", self.name, elapsed_time)
                break
            
            try:
                self._react(plan, procedures)                
            except Exception as ex:
                for p in self.async_procedures:
                    p.cancel()
                raise ex    

            time.sleep(1)


    def _react(self, plan: Plan, procedures: Sequence[Procedure]) -> None:
        """
        Actually run the Procedures and their children if any.
        """
        failed = False
        for p in procedures:
            try:
                if p.is_async():
                    self.async_procedures.append(cast(AsyncProcedure, p))
                p.pre()
                inner_procedures = p.execute()
                self.retries = 0
                if len(inner_procedures) > 0:
                    self._react(plan, inner_procedures)
            except ConfigException as ex:
                plan.report_configuration_error(ex.errorMsg)
                self.cancelAllAsync()
                raise
            except Exception as ex:
                if not failed:
                    self.retries += 1
                    failed = True
                logger.warning("%s Error executing procedure %s rolling back! Error: %s", self.name, p.name, ex)
                try:
                    p.rollback()
                except Exception as rex:
                    logger.warning("%s Error rolling back %s, continue with next procedure. Error: %s", self.name, p.name, rex)
                if p.fail_plan or self.retries >= self.max_retries or isinstance(ex, Ec2InitFatalException):
                    if self.retries >= self.max_retries:
                        logger.error("%s - Reached maximum number of retries. Exiting...", self.name)
                    if isinstance(ex, Ec2InitFatalException):
                        msg = f"{self.name} - Received fatal exception. Reason: {str(ex)}"
                        if ex.__context__:
                            msg = f"{msg} {ex.__context__}"
                        logger.fatal(msg)
                    if p.friendly_name is None:
                        friendly_name = p.name
                    else:
                        friendly_name = p.friendly_name
                    error_message = f"Error executing {friendly_name}"
                    plan.report_error(error_message)
                    self.cancelAllAsync()
                    raise

        for async_p in self.async_procedures:
            async_p.max_retries = self.max_retries
            try:
                async_p.check_unhandled_exception()
            except ConfigException as ex:
                plan.report_configuration_error(ex.errorMsg)
                self.cancelAllAsync()
                raise
            except Exception as ex:
                if async_p.friendly_name is None:
                    friendly_name = async_p.name
                else:
                    friendly_name = async_p.friendly_name
                error_message = f"Error executing {friendly_name}"
                plan.report_error(error_message)
                self.cancelAllAsync()
                raise
